@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2015, imageio contributors
+# Copyright (c) 2014-2018, imageio contributors
 # imageio is distributed under the terms of the (new) BSD License.
 
 # This docstring is used at the index of the documentation pages, and
@@ -8,15 +8,15 @@
 """ 
 Imageio is a Python library that provides an easy interface to read and
 write a wide range of image data, including animated images, volumetric
-data, and scientific formats. It is cross-platform, runs on Python 2.x
-and 3.x, and is easy to install.
+data, and scientific formats. It is cross-platform, runs on Python 2.7
+and 3.4+, and is easy to install.
 
 Main website: http://imageio.github.io
 """
 
 # flake8: noqa
 
-__version__ = '2.2.0'
+__version__ = "2.4.1"
 
 # Load some bits from core
 from .core import FormatManager, RETURN_BYTES
