@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2015, imageio contributors
 # Distributed under the (new) BSD License. See LICENSE.txt for more info.
 
 """ This subpackage provides the core functionality of imageio
@@ -8,7 +7,7 @@
 
 # flake8: noqa
 
-from .util import Image, Dict, asarray, image_as_uint, urlopen
+from .util import Image, Array, Dict, asarray, image_as_uint, urlopen
 from .util import BaseProgressIndicator, StdoutProgressIndicator
 from .util import string_types, text_type, binary_type, IS_PYPY
 from .util import get_platform, appdata_dir, resource_dirs, has_module
