@@ -1,8 +1,7 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2015, imageio contributors
 # imageio is distributed under the terms of the (new) BSD License.
 
-""" 
+"""
 These functions represent imageio's main interface for the user. They
 provide a common API to read and write image data for a large
 variety of formats. All read and write functions accept keyword
@@ -35,6 +34,14 @@ that best suits that file-format.
 
 ----
 
+All read-functions return images as numpy arrays, and have a ``meta``
+attribute; the meta-data dictionary can be accessed with ``im.meta``.
+To make this work, imageio actually makes use of a subclass of
+``np.ndarray``. If needed, the image can be converted to a plain numpy
+array using ``np.asarray(im)``.
+
+----
+
 Supported resource URI's:
 
 All functions described here accept a URI to describe the resource to
@@ -59,6 +66,12 @@ For reading:
 For writing one can also use ``'<bytes>'`` or ``imageio.RETURN_BYTES`` to
 make a write function return the bytes instead of writing to a file.
 
+Note that reading from HTTP and zipfiles works for many formats including
+png and jpeg, but may not work for all formats (some plugins "seek" the
+file object, which HTTP/zip streams do not support). In such a case one
+can download/extract the file first. For HTTP one can use something
+like ``imageio.imread(imageio.core.urlopen(url).read(), '.gif')``.
+
 """
 
 from __future__ import absolute_import, print_function, division
@@ -71,10 +84,10 @@ from .. import formats
 
 def help(name=None):
     """ help(name=None)
-    
+
     Print the documentation of the format specified by name, or a list
-    of supported formats if name is omitted. 
-    
+    of supported formats if name is omitted.
+
     Parameters
     ----------
     name : str
@@ -90,17 +103,17 @@ def help(name=None):
 ## Base functions that return a reader/writer
 
 
-def get_reader(uri, format=None, mode='?', **kwargs):
+def get_reader(uri, format=None, mode="?", **kwargs):
     """ get_reader(uri, format=None, mode='?', **kwargs)
-    
+
     Returns a :class:`.Reader` object which can be used to read data
     and meta data from the specified file.
-    
+
     Parameters
     ----------
-    uri : {str, bytes, file}
-        The resource to load the image from, e.g. a filename, http address or
-        file object, see the docs for more info.
+    uri : {str, pathlib.Path, bytes, file}
+        The resource to load the image from, e.g. a filename, pathlib.Path,
+        http address or file object, see the docs for more info.
     format : str
         The format to use to read the file. By default imageio selects
         the appropriate for you based on the filename and its contents.
@@ -111,37 +124,38 @@ def get_reader(uri, format=None, mode='?', **kwargs):
     kwargs : ...
         Further keyword arguments are passed to the reader. See :func:`.help`
         to see what arguments are available for a particular format.
-    """ 
-    
+    """
+
     # Create request object
-    request = Request(uri, 'r' + mode, **kwargs)
-    
+    request = Request(uri, "r" + mode, **kwargs)
+
     # Get format
     if format is not None:
         format = formats[format]
     else:
         format = formats.search_read_format(request)
     if format is None:
-        raise ValueError('Could not find a format to read the specified file '
-                         'in mode %r' % mode)
-    
+        raise ValueError(
+            "Could not find a format to read the specified file " "in mode %r" % mode
+        )
+
     # Return its reader object
     return format.get_reader(request)
 
 
-def get_writer(uri, format=None, mode='?', **kwargs):
+def get_writer(uri, format=None, mode="?", **kwargs):
     """ get_writer(uri, format=None, mode='?', **kwargs)
-    
+
     Returns a :class:`.Writer` object which can be used to write data
     and meta data to the specified file.
-    
+
     Parameters
     ----------
-    uri : {str, file}
-        The resource to write the image to, e.g. a filename or file object,
-        see the docs for more info.
+    uri : {str, pathlib.Path, file}
+        The resource to write the image to, e.g. a filename, pathlib.Path
+        or file object, see the docs for more info.
     format : str
-        The format to use to read the file. By default imageio selects
+        The format to use to write the file. By default imageio selects
         the appropriate for you based on the filename.
     mode : {'i', 'I', 'v', 'V', '?'}
         Used to give the writer a hint on what the user expects (default '?'):
@@ -150,68 +164,75 @@ def get_writer(uri, format=None, mode='?', **kwargs):
     kwargs : ...
         Further keyword arguments are passed to the writer. See :func:`.help`
         to see what arguments are available for a particular format.
-    """ 
-    
+    """
+
     # Signal extension when returning as bytes, needed by e.g. ffmpeg
     if uri == RETURN_BYTES and isinstance(format, str):
-        uri = RETURN_BYTES + '.' + format.strip('. ')
-    
+        uri = RETURN_BYTES + "." + format.strip(". ")
+
     # Create request object
-    request = Request(uri, 'w' + mode, **kwargs)
-    
+    request = Request(uri, "w" + mode, **kwargs)
+
     # Get format
     if format is not None:
         format = formats[format]
     else:
         format = formats.search_write_format(request)
     if format is None:
-        raise ValueError('Could not find a format to write the specified file '
-                         'in mode %r' % mode)
-    
+        raise ValueError(
+            "Could not find a format to write the specified file " "in mode %r" % mode
+        )
+
     # Return its writer object
     return format.get_writer(request)
 
 
 ## Images
 
+
 def imread(uri, format=None, **kwargs):
     """ imread(uri, format=None, **kwargs)
-    
+
     Reads an image from the specified file. Returns a numpy array, which
     comes with a dict of meta data at its 'meta' attribute.
-    
+
     Note that the image data is returned as-is, and may not always have
     a dtype of uint8 (and thus may differ from what e.g. PIL returns).
-    
+
     Parameters
     ----------
-    uri : {str, bytes, file}
-        The resource to load the image from, e.g. a filename, http address or
-        file object, see the docs for more info.
+    uri : {str, pathlib.Path, bytes, file}
+        The resource to load the image from, e.g. a filename, pathlib.Path,
+        http address or file object, see the docs for more info.
     format : str
         The format to use to read the file. By default imageio selects
         the appropriate for you based on the filename and its contents.
     kwargs : ...
         Further keyword arguments are passed to the reader. See :func:`.help`
         to see what arguments are available for a particular format.
-    """ 
-    
+    """
+
+    if "mode" in kwargs:
+        raise TypeError(
+            'Invalid keyword argument "mode", ' 'perhaps you mean "pilmode"?'
+        )
+
     # Get reader and read first
-    reader = read(uri, format, 'i', **kwargs)
+    reader = read(uri, format, "i", **kwargs)
     with reader:
         return reader.get_data(0)
 
 
 def imwrite(uri, im, format=None, **kwargs):
     """ imwrite(uri, im, format=None, **kwargs)
-    
+
     Write an image to the specified file.
-    
+
     Parameters
     ----------
-    uri : {str, file}
-        The resource to write the image to, e.g. a filename or file object,
-        see the docs for more info.
+    uri : {str, pathlib.Path, file}
+        The resource to write the image to, e.g. a filename, pathlib.Path
+        or file object, see the docs for more info.
     im : numpy.ndarray
         The image data. Must be NxM, NxMx3 or NxMx4.
     format : str
@@ -220,8 +241,8 @@ def imwrite(uri, im, format=None, **kwargs):
     kwargs : ...
         Further keyword arguments are passed to the writer. See :func:`.help`
         to see what arguments are available for a particular format.
-    """ 
-    
+    """
+
     # Test image
     if isinstance(im, np.ndarray):
         if im.ndim == 2:
@@ -229,32 +250,33 @@ def imwrite(uri, im, format=None, **kwargs):
         elif im.ndim == 3 and im.shape[2] in [1, 3, 4]:
             pass
         else:
-            raise ValueError('Image must be 2D (grayscale, RGB, or RGBA).')
+            raise ValueError("Image must be 2D (grayscale, RGB, or RGBA).")
     else:
-        raise ValueError('Image must be a numpy array.')
-    
+        raise ValueError("Image must be a numpy array.")
+
     # Get writer and write first
-    writer = get_writer(uri, format, 'i', **kwargs)
+    writer = get_writer(uri, format, "i", **kwargs)
     with writer:
         writer.append_data(im)
-    
+
     # Return a result if there is any
     return writer.request.get_result()
 
 
 ## Multiple images
 
+
 def mimread(uri, format=None, memtest=True, **kwargs):
     """ mimread(uri, format=None, memtest=True, **kwargs)
-    
+
     Reads multiple images from the specified file. Returns a list of
     numpy arrays, each with a dict of meta data at its 'meta' attribute.
-    
+
     Parameters
     ----------
-    uri : {str, bytes, file}
-        The resource to load the images from, e.g. a filename, http address or
-        file object, see the docs for more info.
+    uri : {str, pathlib.Path, bytes, file}
+        The resource to load the images from, e.g. a filename,pathlib.Path,
+        http address or file object, see the docs for more info.
     format : str
         The format to use to read the file. By default imageio selects
         the appropriate for you based on the filename and its contents.
@@ -267,11 +289,11 @@ def mimread(uri, format=None, memtest=True, **kwargs):
     kwargs : ...
         Further keyword arguments are passed to the reader. See :func:`.help`
         to see what arguments are available for a particular format.
-    """ 
-    
+    """
+
     # Get reader
-    reader = read(uri, format, 'I', **kwargs)
-    
+    reader = read(uri, format, "I", **kwargs)
+
     # Read
     ims = []
     nbytes = 0
@@ -281,23 +303,25 @@ def mimread(uri, format=None, memtest=True, **kwargs):
         nbytes += im.nbytes
         if memtest and nbytes > 256 * 1024 * 1024:
             ims[:] = []  # clear to free the memory
-            raise RuntimeError('imageio.mimread() has read over 256 MiB of '
-                               'image data.\nStopped to avoid memory problems.'
-                               ' Use imageio.get_reader() or memtest=False.')
-    
+            raise RuntimeError(
+                "imageio.mimread() has read over 256 MiB of "
+                "image data.\nStopped to avoid memory problems."
+                " Use imageio.get_reader() or memtest=False."
+            )
+
     return ims
 
 
 def mimwrite(uri, ims, format=None, **kwargs):
     """ mimwrite(uri, ims, format=None, **kwargs)
-    
+
     Write multiple images to the specified file.
-    
+
     Parameters
     ----------
-    uri : {str, file}
-        The resource to write the images to, e.g. a filename or file object,
-        see the docs for more info.
+    uri : {str, pathlib.Path, file}
+        The resource to write the images to, e.g. a filename, pathlib.Path
+        or file object, see the docs for more info.
     ims : sequence of numpy arrays
         The image data. Each array must be NxM, NxMx3 or NxMx4.
     format : str
@@ -306,19 +330,16 @@ def mimwrite(uri, ims, format=None, **kwargs):
     kwargs : ...
         Further keyword arguments are passed to the writer. See :func:`.help`
         to see what arguments are available for a particular format.
-    """ 
-    
-    #Ensure non empty sequence
-    if len(ims) == 0:
-        raise ValueError('Numpy sequence must not be empty.')
-    
+    """
     # Get writer
-    writer = get_writer(uri, format, 'I', **kwargs)
+    writer = get_writer(uri, format, "I", **kwargs)
+    written = 0
+
     with writer:
-        
+
         # Iterate over images (ims may be a generator)
         for im in ims:
-            
+
             # Test image
             if isinstance(im, np.ndarray):
                 if im.ndim == 2:
@@ -326,55 +347,61 @@ def mimwrite(uri, ims, format=None, **kwargs):
                 elif im.ndim == 3 and im.shape[2] in [1, 3, 4]:
                     pass
                 else:
-                    raise ValueError('Image must be 2D '
-                                     '(grayscale, RGB, or RGBA).')
+                    raise ValueError("Image must be 2D " "(grayscale, RGB, or RGBA).")
             else:
-                raise ValueError('Image must be a numpy array.')
-            
+                raise ValueError("Image must be a numpy array.")
+
             # Add image
             writer.append_data(im)
-    
+            written += 1
+
+    # Check that something was written. Check after writing, because ims might
+    # be a generator. The damage is done, but we want to error when it happens.
+    if not written:
+        raise RuntimeError("Zero images were written.")
+
     # Return a result if there is any
     return writer.request.get_result()
 
 
 ## Volumes
 
+
 def volread(uri, format=None, **kwargs):
     """ volread(uri, format=None, **kwargs)
-    
+
     Reads a volume from the specified file. Returns a numpy array, which
     comes with a dict of meta data at its 'meta' attribute.
-    
+
     Parameters
     ----------
-    uri : {str, bytes, file}
-        The resource to load the volume from, e.g. a filename, http address or
-        file object, see the docs for more info.
+    uri : {str, pathlib.Path, bytes, file}
+        The resource to load the volume from, e.g. a filename, pathlib.Path,
+        http address or file object, see the docs for more info.
     format : str
         The format to use to read the file. By default imageio selects
         the appropriate for you based on the filename and its contents.
     kwargs : ...
         Further keyword arguments are passed to the reader. See :func:`.help`
         to see what arguments are available for a particular format.
-    """ 
-    
+    """
+
     # Get reader and read first
-    reader = read(uri, format, 'v', **kwargs)
+    reader = read(uri, format, "v", **kwargs)
     with reader:
         return reader.get_data(0)
 
 
 def volwrite(uri, im, format=None, **kwargs):
     """ volwrite(uri, vol, format=None, **kwargs)
-    
+
     Write a volume to the specified file.
-    
+
     Parameters
     ----------
-    uri : {str, file}
-        The resource to write the image to, e.g. a filename or file object,
-        see the docs for more info.
+    uri : {str, pathlib.Path, file}
+        The resource to write the image to, e.g. a filename, pathlib.Path
+        or file object, see the docs for more info.
     vol : numpy.ndarray
         The image data. Must be NxMxL (or NxMxLxK if each voxel is a tuple).
     format : str
@@ -383,8 +410,8 @@ def volwrite(uri, im, format=None, **kwargs):
     kwargs : ...
         Further keyword arguments are passed to the writer. See :func:`.help`
         to see what arguments are available for a particular format.
-    """ 
-    
+    """
+
     # Test image
     if isinstance(im, np.ndarray):
         if im.ndim == 3:
@@ -392,33 +419,33 @@ def volwrite(uri, im, format=None, **kwargs):
         elif im.ndim == 4 and im.shape[3] < 32:  # How large can a tuple be?
             pass
         else:
-            raise ValueError('Image must be 3D, or 4D if each voxel is '
-                             'a tuple.')
+            raise ValueError("Image must be 3D, or 4D if each voxel is " "a tuple.")
     else:
-        raise ValueError('Image must be a numpy array.')
-    
+        raise ValueError("Image must be a numpy array.")
+
     # Get writer and write first
-    writer = get_writer(uri, format, 'v', **kwargs)
+    writer = get_writer(uri, format, "v", **kwargs)
     with writer:
         writer.append_data(im)
-    
+
     # Return a result if there is any
     return writer.request.get_result()
 
 
 ## Multiple volumes
 
+
 def mvolread(uri, format=None, memtest=True, **kwargs):
     """ mvolread(uri, format=None, memtest=True, **kwargs)
-    
+
     Reads multiple volumes from the specified file. Returns a list of
     numpy arrays, each with a dict of meta data at its 'meta' attribute.
-    
+
     Parameters
     ----------
-    uri : {str, bytes, file}
-        The resource to load the volumes from, e.g. a filename, http address or
-        file object, see the docs for more info.
+    uri : {str, pathlib.Path, bytes, file}
+        The resource to load the volumes from, e.g. a filename, pathlib.Path,
+        http address or file object, see the docs for more info.
     format : str
         The format to use to read the file. By default imageio selects
         the appropriate for you based on the filename and its contents.
@@ -431,11 +458,11 @@ def mvolread(uri, format=None, memtest=True, **kwargs):
     kwargs : ...
         Further keyword arguments are passed to the reader. See :func:`.help`
         to see what arguments are available for a particular format.
-    """ 
-    
+    """
+
     # Get reader and read all
-    reader = read(uri, format, 'V', **kwargs)
-    
+    reader = read(uri, format, "V", **kwargs)
+
     ims = []
     nbytes = 0
     for im in reader:
@@ -444,23 +471,25 @@ def mvolread(uri, format=None, memtest=True, **kwargs):
         nbytes += im.nbytes
         if memtest and nbytes > 1024 * 1024 * 1024:  # pragma: no cover
             ims[:] = []  # clear to free the memory
-            raise RuntimeError('imageio.mvolread() has read over 1 GiB of '
-                               'image data.\nStopped to avoid memory problems.'
-                               ' Use imageio.get_reader() or memtest=False.')
-    
+            raise RuntimeError(
+                "imageio.mvolread() has read over 1 GiB of "
+                "image data.\nStopped to avoid memory problems."
+                " Use imageio.get_reader() or memtest=False."
+            )
+
     return ims
 
 
 def mvolwrite(uri, ims, format=None, **kwargs):
     """ mvolwrite(uri, vols, format=None, **kwargs)
-    
+
     Write multiple volumes to the specified file.
-    
+
     Parameters
     ----------
-    uri : {str, file}
-        The resource to write the volumes to, e.g. a filename or file object,
-        see the docs for more info.
+    uri : {str, pathlib.Path, file}
+        The resource to write the volumes to, e.g. a filename, pathlib.Path
+        or file object, see the docs for more info.
     ims : sequence of numpy arrays
         The image data. Each array must be NxMxL (or NxMxLxK if each
         voxel is a tuple).
@@ -470,15 +499,17 @@ def mvolwrite(uri, ims, format=None, **kwargs):
     kwargs : ...
         Further keyword arguments are passed to the writer. See :func:`.help`
         to see what arguments are available for a particular format.
-    """ 
-    
+    """
+
     # Get writer
-    writer = get_writer(uri, format, 'V', **kwargs)
+    writer = get_writer(uri, format, "V", **kwargs)
+    written = 0
+
     with writer:
-        
+
         # Iterate over images (ims may be a generator)
         for im in ims:
-            
+
             # Test image
             if isinstance(im, np.ndarray):
                 if im.ndim == 3:
@@ -486,14 +517,21 @@ def mvolwrite(uri, ims, format=None, **kwargs):
                 elif im.ndim == 4 and im.shape[3] < 32:
                     pass  # How large can a tuple be?
                 else:
-                    raise ValueError('Image must be 3D, or 4D if each voxel is'
-                                     'a tuple.')
+                    raise ValueError(
+                        "Image must be 3D, or 4D if each voxel is" "a tuple."
+                    )
             else:
-                raise ValueError('Image must be a numpy array.')
-            
+                raise ValueError("Image must be a numpy array.")
+
             # Add image
             writer.append_data(im)
-    
+            written += 1
+
+    # Check that something was written. Check after writing, because ims might
+    # be a generator. The damage is done, but we want to error when it happens.
+    if not written:
+        raise RuntimeError("Zero volumes were written.")
+
     # Return a result if there is any
     return writer.request.get_result()
 
