@@ -7,10 +7,10 @@ import sys
 
 def get_includes():
     if sys.version_info[0] == 3:
-        urllib = ['email', 'urllib.request', ]
+        urllib = ["email", "urllib.request"]
     else:
-        urllib = ['urllib2']
-    return urllib + ['numpy', 'zipfile', 'io']
+        urllib = ["urllib2"]
+    return urllib + ["numpy", "zipfile", "io"]
 
 
 def get_excludes():
