@@ -1,9 +1,8 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2015, imageio contributors
 # imageio is distributed under the terms of the (new) BSD License.
 # This code was taken from visvis/vvmovy/images2swf.py
 
-# styletest: ignore E261 
+# styletest: ignore E261
 
 """ 
 Provides a function (write_swf) to store a series of numpy arrays in an
@@ -47,7 +46,7 @@ import numpy as np
 
 from ..core import string_types, binary_type
 
-PY3 = sys.version_info >= (3, )
+PY3 = sys.version_info >= (3,)
 
 # todo: use Pillow to support reading JPEG images from SWF?
 
@@ -72,14 +71,14 @@ class BitArray:
         return self._len  # self.data.shape[0]
 
     def __repr__(self):
-        return self.data[:self._len].tostring().decode('ascii')
+        return self.data[: self._len].tostring().decode("ascii")
 
     def _checkSize(self):
         # check length... grow if necessary
         arraylen = self.data.shape[0]
         if self._len >= arraylen:
-            tmp = np.zeros((arraylen*2,), dtype=np.uint8)
-            tmp[:self._len] = self.data[:self._len]
+            tmp = np.zeros((arraylen * 2,), dtype=np.uint8)
+            tmp[: self._len] = self.data[: self._len]
             self.data = tmp
 
     def __add__(self, value):
@@ -104,8 +103,8 @@ class BitArray:
 
     def reverse(self):
         """ In-place reverse. """
-        tmp = self.data[:self._len].copy()
-        self.data[:self._len] = tmp[::-1]
+        tmp = self.data[: self._len].copy()
+        self.data[: self._len] = tmp[::-1]
 
     def tobytes(self):
         """ Convert to bytes. If necessary,
@@ -118,12 +117,12 @@ class BitArray:
         while nbytes * 8 < len(bits):
             nbytes += 1
         # pad
-        bits = bits.ljust(nbytes * 8, '0')
+        bits = bits.ljust(nbytes * 8, "0")
 
         # go from bits to bytes
         bb = binary_type()
         for i in range(nbytes):
-            tmp = int(bits[i * 8: (i + 1) * 8], 2)
+            tmp = int(bits[i * 8 : (i + 1) * 8], 2)
             bb += int2uint8(tmp)
 
         # done
@@ -131,15 +130,19 @@ class BitArray:
 
 
 if PY3:
+
     def int2uint32(i):
-        return int(i).to_bytes(4, 'little')
+        return int(i).to_bytes(4, "little")
 
     def int2uint16(i):
-        return int(i).to_bytes(2, 'little')
+        return int(i).to_bytes(2, "little")
 
     def int2uint8(i):
-        return int(i).to_bytes(1, 'little')
+        return int(i).to_bytes(1, "little")
+
+
 else:  # pragma: no cover
+
     def int2uint32(i):
         number = int(i)
         n1, n2, n3, n4 = 1, 256, 256 * 256, 256 * 256 * 256  # noqa
@@ -177,7 +180,7 @@ def int2bits(i, n=None):
     if n is not None:
         if len(bb) > n:  # pragma: no cover
             raise ValueError("int2bits fail: len larger than padlength.")
-        bb = str(bb).rjust(n, '0')
+        bb = str(bb).rjust(n, "0")
 
     # done
     return BitArray(bb)
@@ -185,17 +188,17 @@ def int2bits(i, n=None):
 
 def bits2int(bb, n=8):
     # Init
-    value = ''
+    value = ""
 
     # Get value in bits
     for i in range(len(bb)):
-        b = bb[i:i+1]
+        b = bb[i : i + 1]
         tmp = bin(ord(b))[2:]
-        #value += tmp.rjust(8,'0')
-        value = tmp.rjust(8, '0') + value
+        # value += tmp.rjust(8,'0')
+        value = tmp.rjust(8, "0") + value
 
     # Make decimal
-    return(int(value[:n], 2))
+    return int(value[:n], 2)
 
 
 def get_type_and_len(bb):
@@ -203,14 +206,14 @@ def get_type_and_len(bb):
     Return (type, length, length_of_full_tag)
     """
     # Init
-    value = ''
+    value = ""
 
     # Get first 16 bits
     for i in range(2):
-        b = bb[i:i + 1]
+        b = bb[i : i + 1]
         tmp = bin(ord(b))[2:]
-        #value += tmp.rjust(8,'0')
-        value = tmp.rjust(8, '0') + value
+        # value += tmp.rjust(8,'0')
+        value = tmp.rjust(8, "0") + value
 
     # Get type and length
     type = int(value[:10], 2)
@@ -218,13 +221,13 @@ def get_type_and_len(bb):
     L2 = L + 2
 
     # Long tag header?
-    if L == 63: # '111111'
-        value = ''
+    if L == 63:  # '111111'
+        value = ""
         for i in range(2, 6):
-            b = bb[i:i + 1]  # becomes a single-byte bytes() on both PY3 & PY2
+            b = bb[i : i + 1]  # becomes a single-byte bytes() on both PY3 & PY2
             tmp = bin(ord(b))[2:]
-            #value += tmp.rjust(8,'0')
-            value = tmp.rjust(8, '0') + value
+            # value += tmp.rjust(8,'0')
+            value = tmp.rjust(8, "0") + value
         L = int(value, 2)
         L2 = L + 6
 
@@ -252,15 +255,15 @@ def signedint2bits(i, n=None):
     bb.reverse()
 
     # justify
-    bb = '0' + str(bb) # always need the sign bit in front
+    bb = "0" + str(bb)  # always need the sign bit in front
     if n is not None:
         if len(bb) > n:  # pragma: no cover
             raise ValueError("signedint2bits fail: len larger than padlength.")
-        bb = bb.rjust(n, '0')
+        bb = bb.rjust(n, "0")
 
     # was it negative? (then opposite bits)
     if i < 0:
-        bb = bb.replace('0', 'x').replace('1', '0').replace('x', '1')
+        bb = bb.replace("0", "x").replace("1", "0").replace("x", "1")
 
     # done
     return BitArray(bb)
@@ -277,7 +280,7 @@ def twits2bits(arr):
     # first determine length using non justified bit strings
     maxlen = 1
     for i in arr:
-        tmp = len(signedint2bits(i*20))
+        tmp = len(signedint2bits(i * 20))
         if tmp > maxlen:
             maxlen = tmp
 
@@ -295,7 +298,7 @@ def floats2bits(arr):
     Negative numbers are not (yet) possible, because I don't
     know how the're implemented (ambiguity).
     """
-    bits = int2bits(31, 5) # 32 does not fit in 5 bits!
+    bits = int2bits(31, 5)  # 32 does not fit in 5 bits!
     for i in arr:
         if i < 0:  # pragma: no cover
             raise ValueError("Dit not implement negative floats!")
@@ -308,8 +311,8 @@ def floats2bits(arr):
 
 ## Base Tag
 
-class Tag:
 
+class Tag:
     def __init__(self):
         self.bytes = binary_type()
         self.tagtype = -1
@@ -326,7 +329,7 @@ class Tag:
         bits = int2bits(self.tagtype, 10)
 
         # complete header uint16 thing
-        bits += '1' * 6  # = 63 = 0x3f
+        bits += "1" * 6  # = 63 = 0x3f
         # make uint16
         bb = int2uint16(int(str(bits), 2))
 
@@ -346,24 +349,24 @@ class Tag:
 
         # empty matrix?
         if scale_xy is None and rot_xy is None and trans_xy is None:
-            return "0"*8
+            return "0" * 8
 
         # init
         bits = BitArray()
 
         # scale
         if scale_xy:
-            bits += '1'
+            bits += "1"
             bits += floats2bits([scale_xy[0], scale_xy[1]])
         else:
-            bits += '0'
+            bits += "0"
 
         # rotation
         if rot_xy:
-            bits += '1'
+            bits += "1"
             bits += floats2bits([rot_xy[0], rot_xy[1]])
         else:
-            bits += '0'
+            bits += "0"
 
         # translation (no flag here)
         if trans_xy:
@@ -377,6 +380,7 @@ class Tag:
 
 ## Control tags
 
+
 class ControlTag(Tag):
     def __init__(self):
         Tag.__init__(self)
@@ -388,7 +392,7 @@ class FileAttributesTag(ControlTag):
         self.tagtype = 69
 
     def process_tag(self):
-        self.bytes = '\x00'.encode('ascii') * (1+3)
+        self.bytes = "\x00".encode("ascii") * (1 + 3)
 
 
 class ShowFrameTag(ControlTag):
@@ -402,6 +406,7 @@ class ShowFrameTag(ControlTag):
 
 class SetBackgroundTag(ControlTag):
     """ Set the color in 0-255, or 0-1 (if floats given). """
+
     def __init__(self, *rgb):
         self.tagtype = 9
         if len(rgb) == 1:
@@ -419,7 +424,7 @@ class SetBackgroundTag(ControlTag):
 
 
 class DoActionTag(Tag):
-    def __init__(self, action='stop'):
+    def __init__(self, action="stop"):
         Tag.__init__(self)
         self.tagtype = 12
         self.actions = [action]
@@ -432,10 +437,10 @@ class DoActionTag(Tag):
 
         for action in self.actions:
             action = action.lower()
-            if action == 'stop':
-                bb += '\x07'.encode('ascii')
-            elif action == 'play':  # pragma: no cover - not used
-                bb += '\x06'.encode('ascii')
+            if action == "stop":
+                bb += "\x07".encode("ascii")
+            elif action == "play":  # pragma: no cover - not used
+                bb += "\x06".encode("ascii")
             else:  # pragma: no cover
                 print("warning, unkown action: %s" % action)
 
@@ -454,10 +459,9 @@ class DefinitionTag(Tag):
 
 
 class BitmapTag(DefinitionTag):
-
     def __init__(self, im):
         DefinitionTag.__init__(self)
-        self.tagtype = 36 # DefineBitsLossless2
+        self.tagtype = 36  # DefineBitsLossless2
 
         # convert image (note that format is ARGB)
         # even a grayscale image is stored in ARGB, nevertheless,
@@ -467,8 +471,7 @@ class BitmapTag(DefinitionTag):
 
         if len(im.shape) == 3:
             if im.shape[2] in [3, 4]:
-                tmp = np.ones((im.shape[0], im.shape[1], 4),
-                              dtype=np.uint8) * 255
+                tmp = np.ones((im.shape[0], im.shape[1], 4), dtype=np.uint8) * 255
                 for i in range(3):
                     tmp[:, :, i + 1] = im[:, :, i]
                 if im.shape[2] == 4:
@@ -477,7 +480,7 @@ class BitmapTag(DefinitionTag):
                 raise ValueError("Invalid shape to be an image.")
 
         elif len(im.shape) == 2:
-            tmp = np.ones((im.shape[0], im.shape[1], 4), dtype=np.uint8)*255
+            tmp = np.ones((im.shape[0], im.shape[1], 4), dtype=np.uint8) * 255
             for i in range(3):
                 tmp[:, :, i + 1] = im[:, :]
         else:  # pragma: no cover
@@ -492,11 +495,11 @@ class BitmapTag(DefinitionTag):
 
         # build tag
         bb = binary_type()
-        bb += int2uint16(self.id)   # CharacterID
-        bb += int2uint8(5)     # BitmapFormat
-        bb += int2uint16(self.imshape[1])   # BitmapWidth
-        bb += int2uint16(self.imshape[0])   # BitmapHeight
-        bb += self._data            # ZlibBitmapData
+        bb += int2uint16(self.id)  # CharacterID
+        bb += int2uint8(5)  # BitmapFormat
+        bb += int2uint16(self.imshape[1])  # BitmapWidth
+        bb += int2uint16(self.imshape[0])  # BitmapHeight
+        bb += self._data  # ZlibBitmapData
 
         self.bytes = bb
 
@@ -519,13 +522,13 @@ class PlaceObjectTag(ControlTag):
         # build PlaceObject2
         bb = binary_type()
         if self.move:
-            bb += '\x07'.encode('ascii')
+            bb += "\x07".encode("ascii")
         else:
             # (8 bit flags): 4:matrix, 2:character, 1:move
-            bb += '\x06'.encode('ascii')  
-        bb += int2uint16(depth) # Depth
-        bb += int2uint16(id) # character id
-        bb += self.make_matrix_record(trans_xy=xy).tobytes() # MATRIX record
+            bb += "\x06".encode("ascii")
+        bb += int2uint16(depth)  # Depth
+        bb += int2uint16(id)  # character id
+        bb += self.make_matrix_record(trans_xy=xy).tobytes()  # MATRIX record
         self.bytes = bb
 
 
@@ -550,32 +553,31 @@ class ShapeTag(DefinitionTag):
 
         # first entry: FILLSTYLEARRAY with in it a single fill style
         bb += int2uint8(1)  # FillStyleCount
-        bb += '\x41'.encode('ascii')  # FillStyleType (0x41 or 0x43 unsmoothed)
+        bb += "\x41".encode("ascii")  # FillStyleType (0x41 or 0x43 unsmoothed)
         bb += int2uint16(self.bitmapId)  # BitmapId
-        #bb += '\x00' # BitmapMatrix (empty matrix with leftover bits filled)
+        # bb += '\x00' # BitmapMatrix (empty matrix with leftover bits filled)
         bb += self.make_matrix_record(scale_xy=(20, 20)).tobytes()
 
-#         # first entry: FILLSTYLEARRAY with in it a single fill style
-#         bb += int2uint8(1)  # FillStyleCount
-#         bb += '\x00' # solid fill
-#         bb += '\x00\x00\xff' # color
+        #         # first entry: FILLSTYLEARRAY with in it a single fill style
+        #         bb += int2uint8(1)  # FillStyleCount
+        #         bb += '\x00' # solid fill
+        #         bb += '\x00\x00\xff' # color
 
         # second entry: LINESTYLEARRAY with a single line style
         bb += int2uint8(0)  # LineStyleCount
-        #bb += int2uint16(0*20) # Width
-        #bb += '\x00\xff\x00'  # Color
+        # bb += int2uint16(0*20) # Width
+        # bb += '\x00\xff\x00'  # Color
 
         # third and fourth entry: NumFillBits and NumLineBits (4 bits each)
         # I each give them four bits, so 16 styles possible.
-        bb += '\x44'.encode('ascii')
+        bb += "\x44".encode("ascii")
 
         self.bytes = bb
 
         # last entries: SHAPERECORDs ... (individual shape records not aligned)
         # STYLECHANGERECORD
         bits = BitArray()
-        bits += self.make_style_change_record(0, 1, 
-                                              moveTo=(self.wh[0], self.wh[1]))
+        bits += self.make_style_change_record(0, 1, moveTo=(self.wh[0], self.wh[1]))
         # STRAIGHTEDGERECORD 4x
         bits += self.make_straight_edge_record(-self.wh[0], 0)
         bits += self.make_straight_edge_record(0, -self.wh[1])
@@ -588,31 +590,30 @@ class ShapeTag(DefinitionTag):
         self.bytes += bits.tobytes()
 
         # done
-        #self.bytes = bb
+        # self.bytes = bb
 
-    def make_style_change_record(self, lineStyle=None, fillStyle=None,
-                                 moveTo=None):
+    def make_style_change_record(self, lineStyle=None, fillStyle=None, moveTo=None):
 
         # first 6 flags
         # Note that we use FillStyle1. If we don't flash (at least 8) does not
         # recognize the frames properly when importing to library.
 
         bits = BitArray()
-        bits += '0'  # TypeFlag (not an edge record)
-        bits += '0'  # StateNewStyles (only for DefineShape2 and Defineshape3)
+        bits += "0"  # TypeFlag (not an edge record)
+        bits += "0"  # StateNewStyles (only for DefineShape2 and Defineshape3)
         if lineStyle:
-            bits += '1'  # StateLineStyle
+            bits += "1"  # StateLineStyle
         else:
-            bits += '0'
+            bits += "0"
         if fillStyle:
-            bits += '1'  # StateFillStyle1
+            bits += "1"  # StateFillStyle1
         else:
-            bits += '0'
-        bits += '0'  # StateFillStyle0
+            bits += "0"
+        bits += "0"  # StateFillStyle0
         if moveTo:
-            bits += '1'  # StateMoveTo
+            bits += "1"  # StateMoveTo
         else:
-            bits += '0'
+            bits += "0"
 
         # give information
         # todo: nbits for fillStyle and lineStyle is hard coded.
@@ -636,9 +637,9 @@ class ShapeTag(DefinitionTag):
         nbits = max([len(xbits), len(ybits)])
 
         bits = BitArray()
-        bits += '11'  # TypeFlag and StraightFlag
-        bits += int2bits(nbits-2, 4)
-        bits += '1'  # GeneralLineFlag
+        bits += "11"  # TypeFlag and StraightFlag
+        bits += int2bits(nbits - 2, 4)
+        bits += "1"  # GeneralLineFlag
         bits += signedint2bits(dxdy[0] * 20, nbits)
         bits += signedint2bits(dxdy[1] * 20, nbits)
 
@@ -648,23 +649,23 @@ class ShapeTag(DefinitionTag):
 
     def make_end_shape_record(self):
         bits = BitArray()
-        bits += "0"     # TypeFlag: no edge
-        bits += "0"*5   # EndOfShape
+        bits += "0"  # TypeFlag: no edge
+        bits += "0" * 5  # EndOfShape
         return bits
 
 
 def read_pixels(bb, i, tagType, L1):
     """ With pf's seed after the recordheader, reads the pixeldata.
     """
-    
+
     # Get info
-    charId = bb[i:i + 2]  # noqa
+    charId = bb[i : i + 2]  # noqa
     i += 2
-    format = ord(bb[i:i + 1])
+    format = ord(bb[i : i + 1])
     i += 1
-    width = bits2int(bb[i:i + 2], 16)
+    width = bits2int(bb[i : i + 2], 16)
     i += 2
-    height = bits2int(bb[i:i + 2], 16)
+    height = bits2int(bb[i : i + 2], 16)
     i += 2
 
     # If we can, get pixeldata and make numpy array
@@ -673,7 +674,7 @@ def read_pixels(bb, i, tagType, L1):
     else:
         # Read byte data
         offset = 2 + 1 + 2 + 2  # all the info bits
-        bb2 = bb[i:i+(L1-offset)]
+        bb2 = bb[i : i + (L1 - offset)]
 
         # Decompress and make numpy array
         data = zlib.decompress(bb2)
@@ -707,6 +708,7 @@ def read_pixels(bb, i, tagType, L1):
 # These are the original public functions, we don't use them, but we
 # keep it so that in principle this module can be used stand-alone.
 
+
 def checkImages(images):  # pragma: no cover
     """ checkImages(images)
     Check numpy images and correct intensity range etc.
@@ -719,11 +721,11 @@ def checkImages(images):  # pragma: no cover
         if isinstance(im, np.ndarray):
             # Check and convert dtype
             if im.dtype == np.uint8:
-                images2.append(im) # Ok
+                images2.append(im)  # Ok
             elif im.dtype in [np.float32, np.float64]:
                 theMax = im.max()
-                if theMax > 128 and theMax < 300:
-                    pass # assume 0:255
+                if 128 < theMax < 300:
+                    pass  # assume 0:255
                 else:
                     im = im.copy()
                     im[im < 0] = 0
@@ -735,29 +737,30 @@ def checkImages(images):  # pragma: no cover
                 images2.append(im)
             # Check size
             if im.ndim == 2:
-                pass # ok
+                pass  # ok
             elif im.ndim == 3:
                 if im.shape[2] not in [3, 4]:
-                    raise ValueError('This array can not represent an image.')
+                    raise ValueError("This array can not represent an image.")
             else:
-                raise ValueError('This array can not represent an image.')
+                raise ValueError("This array can not represent an image.")
         else:
-            raise ValueError('Invalid image type: ' + str(type(im)))
+            raise ValueError("Invalid image type: " + str(type(im)))
 
     # Done
     return images2
 
 
-def build_file(fp, taglist, nframes=1, framesize=(500, 500), fps=10, 
-               version=8):   # pragma: no cover
+def build_file(
+    fp, taglist, nframes=1, framesize=(500, 500), fps=10, version=8
+):  # pragma: no cover
     """ Give the given file (as bytes) a header. """
 
     # compose header
     bb = binary_type()
-    bb += 'F'.encode('ascii')  # uncompressed
-    bb += 'WS'.encode('ascii')  # signature bytes
+    bb += "F".encode("ascii")  # uncompressed
+    bb += "WS".encode("ascii")  # signature bytes
     bb += int2uint8(version)  # version
-    bb += '0000'.encode('ascii')  # FileLength (leave open for now)
+    bb += "0000".encode("ascii")  # FileLength (leave open for now)
     bb += Tag().make_rect_record(0, framesize[0], 0, framesize[1]).tobytes()
     bb += int2uint8(0) + int2uint8(fps)  # FrameRate
     bb += int2uint16(nframes)
@@ -768,7 +771,7 @@ def build_file(fp, taglist, nframes=1, framesize=(500, 500), fps=10,
         fp.write(tag.get_tag())
 
     # finish with end tag
-    fp.write('\x00\x00'.encode('ascii'))
+    fp.write("\x00\x00".encode("ascii"))
 
     # set size
     sze = fp.tell()
@@ -786,7 +789,7 @@ def write_swf(filename, images, duration=0.1, repeat=True):  # pragma: no cover
     0 and 255 for integer types, and between 0 and 1 for float types.
 
     """
-    
+
     # Check images
     images2 = checkImages(images)
 
@@ -794,7 +797,7 @@ def write_swf(filename, images, duration=0.1, repeat=True):  # pragma: no cover
     taglist = [FileAttributesTag(), SetBackgroundTag(0, 0, 0)]
 
     # Check duration
-    if hasattr(duration, '__len__'):
+    if hasattr(duration, "__len__"):
         if len(duration) == len(images2):
             duration = [d for d in duration]
         else:
@@ -804,14 +807,14 @@ def write_swf(filename, images, duration=0.1, repeat=True):  # pragma: no cover
 
     # Build delays list
     minDuration = float(min(duration))
-    delays = [round(d/minDuration) for d in duration]
+    delays = [round(d / minDuration) for d in duration]
     delays = [max(1, int(d)) for d in delays]
 
     # Get FPS
-    fps = 1.0/minDuration
+    fps = 1.0 / minDuration
 
     # Produce series of tags for each image
-    #t0 = time.time()
+    # t0 = time.time()
     nframes = 0
     for im in images2:
         bm = BitmapTag(im)
@@ -824,20 +827,20 @@ def write_swf(filename, images, duration=0.1, repeat=True):  # pragma: no cover
         nframes += 1
 
     if not repeat:
-        taglist.append(DoActionTag('stop'))
+        taglist.append(DoActionTag("stop"))
 
     # Build file
-    #t1 = time.time()
-    fp = open(filename, 'wb')
+    # t1 = time.time()
+    fp = open(filename, "wb")
     try:
         build_file(fp, taglist, nframes=nframes, framesize=wh, fps=fps)
     except Exception:
         raise
     finally:
         fp.close()
-    #t2 = time.time()
+    # t2 = time.time()
 
-    #print("Writing SWF took %1.2f and %1.2f seconds" % (t1-t0, t2-t1) )
+    # print("Writing SWF took %1.2f and %1.2f seconds" % (t1-t0, t2-t1) )
 
 
 def read_swf(filename):  # pragma: no cover
@@ -849,35 +852,35 @@ def read_swf(filename):  # pragma: no cover
 
     # Check whether it exists
     if not os.path.isfile(filename):
-        raise IOError('File not found: '+str(filename))
+        raise IOError("File not found: " + str(filename))
 
     # Init images
     images = []
 
     # Open file and read all
-    fp = open(filename, 'rb')
+    fp = open(filename, "rb")
     bb = fp.read()
 
     try:
         # Check opening tag
-        tmp = bb[0:3].decode('ascii', 'ignore')
-        if tmp.upper() == 'FWS':
-            pass # ok
-        elif tmp.upper() == 'CWS':
+        tmp = bb[0:3].decode("ascii", "ignore")
+        if tmp.upper() == "FWS":
+            pass  # ok
+        elif tmp.upper() == "CWS":
             # Decompress movie
             bb = bb[:8] + zlib.decompress(bb[8:])
         else:
-            raise IOError('Not a valid SWF file: ' + str(filename))
+            raise IOError("Not a valid SWF file: " + str(filename))
 
         # Set filepointer at first tag (skipping framesize RECT and two uin16's
         i = 8
-        nbits = bits2int(bb[i: i + 1], 5)  # skip FrameSize
+        nbits = bits2int(bb[i : i + 1], 5)  # skip FrameSize
         nbits = 5 + nbits * 4
         Lrect = nbits / 8.0
         if Lrect % 1:
             Lrect += 1
         Lrect = int(Lrect)
-        i += Lrect+4
+        i += Lrect + 4
 
         # Iterate over the tags
         counter = 0
@@ -885,26 +888,26 @@ def read_swf(filename):  # pragma: no cover
             counter += 1
 
             # Get tag header
-            head = bb[i:i+6]
+            head = bb[i : i + 6]
             if not head:
-                break # Done (we missed end tag)
+                break  # Done (we missed end tag)
 
             # Determine type and length
             T, L1, L2 = get_type_and_len(head)
             if not L2:
-                print('Invalid tag length, could not proceed')
+                print("Invalid tag length, could not proceed")
                 break
-            #print(T, L2)
+            # print(T, L2)
 
             # Read image if we can
             if T in [20, 36]:
-                im = read_pixels(bb, i+6, T, L1)
+                im = read_pixels(bb, i + 6, T, L1)
                 if im is not None:
                     images.append(im)
             elif T in [6, 21, 35, 90]:
-                print('Ignoring JPEG image: cannot read JPEG.')
+                print("Ignoring JPEG image: cannot read JPEG.")
             else:
-                pass # Not an image tag
+                pass  # Not an image tag
 
             # Detect end tag
             if T == 0:
@@ -915,7 +918,7 @@ def read_swf(filename):  # pragma: no cover
 
     finally:
         fp.close()
-    
+
     # Done
     return images
 
