@@ -20,30 +20,30 @@ except ImportError:
     itk = None
 
 
-@pytest.mark.skipif('itk is None')
+@pytest.mark.skipif("itk is None")
 def test_simpleitk_reading_writing():
     """ Test reading and saveing tiff """
     im2 = np.ones((10, 10, 3), np.uint8) * 2
 
-    filename1 = os.path.join(test_dir, 'test_tiff.tiff')
+    filename1 = os.path.join(test_dir, "test_tiff.tiff")
 
     # One image
-    imageio.imsave(filename1, im2, 'itk')
-    im = imageio.imread(filename1, 'itk')
-    ims = imageio.mimread(filename1, 'itk')
+    imageio.imsave(filename1, im2, "itk")
+    im = imageio.imread(filename1, "itk")
+    ims = imageio.mimread(filename1, "itk")
     assert (im == im2).all()
     assert len(ims) == 1
 
     # Mixed
-    W = imageio.save(filename1, 'itk')
+    W = imageio.save(filename1, "itk")
     raises(RuntimeError, W.set_meta_data, 1)
-    assert W.format.name == 'ITK'
+    assert W.format.name == "ITK"
     W.append_data(im2)
     W.append_data(im2)
     W.close()
     #
-    R = imageio.read(filename1, 'itk')
-    assert R.format.name == 'ITK'
+    R = imageio.read(filename1, "itk")
+    assert R.format.name == "ITK"
     ims = list(R)  # == [im for im in R]
     assert (ims[0] == im2).all()
     # Fail
